#include <curl/curl.h>
#include <curl/easy.h>
#include <stdlib.h>
#include <string.h>

#include "include/curl.h"

#define API_URL "https://openlibrary.org/search.json?page=1&q="
#define API_URL_LEN 45

static void setup_api_link(const char* isbn, char** buf)
{
	size_t buffer_len;
	
	buffer_len = API_URL_LEN + strlen(isbn);
	(*buf) = malloc(sizeof(char) * buffer_len);

	strncpy((*buf), API_URL, API_URL_LEN + 1);
	strncpy((*buf)+ API_URL_LEN, isbn, strlen(isbn));
	(*buf)[buffer_len] = '\0';

}

#undef API_URL
#undef API_URL_LEN

static void init_get_response(get_response* buf)
{
	buf->len = 0;
	buf->buf = malloc(buf->len + 1);
	if (!(buf->buf)) {
		perror("malloc");
		exit(EXIT_FAILURE);
	}
	buf->buf[0] = '\0';
}

void free_get_response(get_response* buf)
{
	free(buf->buf);
}

static size_t curl_write_response(void* ptr, size_t size, size_t nmemb, get_response* buf)
{
	size_t new_len = buf->len + (size * nmemb);
	buf->buf = realloc(buf->buf, new_len + 1);
	if (!(buf->buf)) {
		perror("realloc");
		exit(EXIT_FAILURE);
	}
	memcpy(buf->buf + buf->len, ptr, size * nmemb);
	buf->buf[new_len] = '\0';
	buf->len = new_len;
	return size * nmemb;
}

CURLcode book_api_call(const char* isbn, get_response* buf)
{
	CURL* handler;
	CURLcode result;
	char* request_url;

	handler = curl_easy_init();
	if (!handler) {
		perror("curl_easy_init");
		exit(EXIT_FAILURE);
	}

	setup_api_link(isbn, &request_url);
	init_get_response(buf);

	curl_easy_setopt(handler, CURLOPT_URL, request_url);
	curl_easy_setopt(handler, CURLOPT_HTTPGET, 1L);
	curl_easy_setopt(handler, CURLOPT_WRITEFUNCTION, curl_write_response);
	curl_easy_setopt(handler, CURLOPT_WRITEDATA, buf);

	result = curl_easy_perform(handler);

	curl_easy_cleanup(handler);

	return result;
}


