#include <cjson/cJSON.h>
#include <argp.h>
#include <stdlib.h>
#include <stdbool.h>

#include "include/curl.h"
#include "include/db.h"
#include "include/json.h"

struct flags {
	char* isbn;
	bool json, db, print;
};

static void init_flags(struct flags* flags) {flags->json = false;flags->db = false;flags->print = false;}

const char* argp_program_version = "isbn-lookup 2.1";
const char* argp_program_bug_address = "supernovaa41@gmx.com";
static char doc[] = "Lookup details of a book from the ISBN. Using OpenLibrary API.";
static char args_doc[] = "<ISBN>";
static struct argp_option options[] = {
	{"json", 'j', 0, 0, "output to json", 0},
	{"db", 'd', 0, 0, "output to sqlite3 database", 0},
	{0, 'p', 0, 0, "print the full json response", 0},
	{0}
};

static error_t parse_opt(int key, char* arg, struct argp_state* state)
{
	struct flags* flags = state->input;
	switch (key) {
		case 'j':
			if (flags->db) {
				fprintf(stderr, "-j and -d are mutually exclusive.\n");
				exit(EXIT_FAILURE);
			}
			flags->json = true;
			break;
		case 'd':
			if (flags->json) {
				fprintf(stderr, "-j and -d are mutually exclusive.\n");
				exit(EXIT_FAILURE);
			}
			flags->db = true;
			break;
		case 'p':
			flags->print = true;
			break;
		case ARGP_KEY_ARG:
			flags->isbn = arg;
			break;
		case ARGP_KEY_END:
			if (state->arg_num < 1)
				argp_usage(state);
			break;
		default:
			return ARGP_ERR_UNKNOWN;
	}
	return 0;
}

static struct argp argp = {options, parse_opt, args_doc, doc, 0, 0, 0};
struct flags flags;

int main(int argc, char* argv[])
{
	get_response resp;
	cJSON* query;

	init_flags(&flags);
	argp_parse(&argp, argc, argv, 0, 0, &flags);

	/*
	//book_api_call("9780762437818", &resp);
	book_api_call(flags.isbn, &resp);

	if (flags.print)
		puts(resp.buf);

	get_json_book(&resp, &query);
	free_get_response(&resp);

	if (flags.json)
		json_out(&query);
	else
	 	;	// db out;
	*/

	book_t book;

	book.title = "hi";
	book.subtitle = "test";
	book.isbn = "1234";
	book.authors = "test";
	book.subjects = "test";
	book.langs = "eng";
	book.date_added = "test";
	book.pub_date = "test";
	book.image_code = "test";

	book.page_len = 1;

	sqlite3* db;

	setup_db(db);

	add_to_db(&book, db);

	
	return 0;
}
